/**
   An Environment object holds <variable,value> pairs.

   The Environment interface has methods for
   1) Adding new <variable,value> pairs to an environment object.
   2) Looking up a variable to see if it is part of a <variable,value> pair.
   3) Looking up a variable's value from the variable's <variable,value> pair.
   4) Mutating the value part of a varaible's <variable,value> pair.
*/
import java.util.ArrayList;

public class Environment
{
   private ArrayList<String> variables;
   private ArrayList<Value>  values;


   /**
      Default contructor creates an empty Environment object.
   */
   public Environment()
   {
      variables = new ArrayList<String>();
      values    = new ArrayList<Value>();
   }


   /**
      Add a <variable, value> pair to this environment object.
   */
   public void add(String variable, Value value)
   {
      variables.add(variable);
         values.add(value);
   }


   /**
      Look up variable in the environment.
      Return true if the variable is in it,
      otherwise return false.
   */
   public boolean defined(String variable)
   {
      return ( null != this.lookUp(variable) );
   }


   /**
      Look up variable in the environment and
      return its associated value.

      Return null if variable is not found.
   */
   public Value lookUp(String variable)
   {
      int i;
      for (i = 0; i < variables.size(); i++)
         if ( variable.equals(variables.get(i)) )
            break;

      if ( i < variables.size() )
      {
         return values.get(i);
      }
      else
      {
         return null;  // variable cannot be found
      }
   }


   /**
      Update the value associated with variable in this environment.
      Return true if the update is succesfull,
      return false if the variable is not found.
   */
   public boolean update(String variable, Value value)
   {
      int i;
      for (i = 0; i < variables.size(); i++)
         if ( variable.equals(variables.get(i)) )
            break;

      if ( i < variables.size() )
      {
         values.set(i, value);
         return true;
      }
      else
      {
         return false;
      }
   }


   /**
      Convert the contents of the environment into a string.
      This is mainly for debugging purposes.
   */
   public String toString()
   {
      String result = "";

      result += "[Global Environment";

      for (int i = 0; i < variables.size(); i++)
      {
         result += "\n " + variables.get(i) + " = " + values.get(i);
      }
      result += "]";

      return result;
   }

}//Environment